--------------------------------------------------------------------------------
-- Weapon Freeze Gun + Projectile Ray
-- Original Carnage Contest Weapon
-- Script by DC, August 2011, www.UnrealSoftware.de
--------------------------------------------------------------------------------

-- Setup Tables
if cc==nil then cc={} end
cc.freezegun={}
cc.freezegun.ray={}

-- Load & Prepare Ressources
cc.freezegun.gfx_wpn=loadgfx("weapons/freezegun.bmp")				-- Weapon Image
setmidhandle(cc.freezegun.gfx_wpn)
cc.freezegun.gfx_pro=loadgfx("weapons/lasershot.png")				-- Projectile Image
setmidhandle(cc.freezegun.gfx_pro)
cc.freezegun.gfx_ice=loadgfx("weapons/ice.bmp")						-- Impact Ice
cc.freezegun.sfx_attack=loadsfx("laser.wav")						-- Attack Sound
cc.freezegun.sfx_impact=loadsfx("laserimpact.wav")					-- Impact Sound
cc.freezegun.sfx_freeze=loadsfx("freeze.wav")						-- Freeze Sound

--------------------------------------------------------------------------------
-- Weapon: Freeze Gun
--------------------------------------------------------------------------------

cc.freezegun.id=addweapon("cc.freezegun","Freeze Gun",cc.freezegun.gfx_wpn,0,2)	-- Add Weapon (0 uses, first in round 2)
addweaponinfo("Can freeze players in 3 separate shots. Creates ice crystals when it hits water. Removes fire. Frozen players are unable to move or to turn around (for one turn).")
cc.freezegun.ammo=3													-- 3 Shots

function cc.freezegun.draw()										-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(cc.freezegun.gfx_wpn,6,0)
	-- HUD ammobar
	if cc.freezegun.ammo-weapon_shots>0 then
		hudammobar(cc.freezegun.ammo-weapon_shots,cc.freezegun.ammo)
	end
	-- HUD Crosshair
	if cc.freezegun.ammo-weapon_shots>0 then
		hudcrosshair(7,3)
	end
end

function cc.freezegun.attack(attack)								-- Attack
	-- Decrement timer
	if weapon_timer>0 then
		weapon_timer=weapon_timer-1
	end
	-- Attack
	if weapon_shots<cc.freezegun.ammo and weapon_timer<=0 and attack==1 then
		-- No more weapon switching!
		useweapon(0)
		-- Reset Timer
		weapon_timer=15
		-- Attack
		playsound(cc.freezegun.sfx_attack)
		weapon_shots=weapon_shots+1
		id=createprojectile(cc.freezegun.ray.id)
		projectiles[id]={}
		-- Ignore collision with current player at beginning
		projectiles[id].ignore=playercurrent()
		-- Set initial position of projectile
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(math.rad(getplayerrotation(0)))*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(math.rad(getplayerrotation(0)))*5.0
		-- Set speed of projectile
		projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*6.0
		projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*6.0
		-- Timer
		projectiles[id].timer=0
		-- Initial movement
		projectiles[id].x=projectiles[id].x-projectiles[id].sx*1.5
		projectiles[id].y=projectiles[id].y-projectiles[id].sy*1.5
		for i=1,3,1 do
			if cc.freezegun.ray.move(id)==1 then
				break
			end
		end
		-- Effects
		recoil(3)
		particle(p_muzzle,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlecolor(230,230,255)
		particlefadealpha(0.01)
		particle(p_smoke,getplayerx(0)+(getplayerdirection(0)*7)+math.sin(math.rad(getplayerrotation(0)))*12,getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*12)
		particlespeed(-0.2+math.random()*0.4+getwind()*10.0,-1.0+math.random()*0.6)
		particlefadealpha(0.005)
		-- End Turn
		if (weapon_shots>=cc.freezegun.ammo) then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Ray
--------------------------------------------------------------------------------

cc.freezegun.ray.id=addprojectile("cc.freezegun.ray")			-- Add Projectile

function cc.freezegun.ray.draw(id)								-- Draw
	-- Setup draw mode
	setblend(blend_light)
	setalpha(1)
	setcolor(230,230,255)
	setscale(1.0,0.5)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(cc.freezegun.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function cc.freezegun.ray.update(id)							-- Update
	-- Particle Tail
	particle(p_lightpuff,projectiles[id].x,projectiles[id].y)
	particlespeed(math.random(-5,5)*0.1,math.random(-5,5)*0.1)
	particlefadealpha(0.05)
	particlesize(math.random(20,40)*0.1,math.random(20,40)*0.1)
	particlecolor(230,230,255)
	-- Move
	cc.freezegun.ray.move(id)
end

function cc.freezegun.ray.move(id)
	-- Timer (free after 10 secs)
	projectiles[id].timer=projectiles[id].timer+1
	if projectiles[id].timer>500 then
		-- Free projectile
		freeprojectile(id)
		return 1
	end
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby		
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*5,projectiles[id].y-math.cos(math.rad(rot))*5)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore then
				-- Freeze player
				if playercollision()~=0 then
					playsound(cc.freezegun.sfx_freeze)
					playerstate(playercollision(),state_frozen,1)
				end
				-- Sound
				playsound(cc.freezegun.sfx_impact)
				-- Crater
				blue=math.random(200,255)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
				else
					terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
				end
				-- Particles
				particle(p_muzzle,projectiles[id].x,projectiles[id].y)
				particlecolor(255,255,255)
				for angle=0,350,10 do
					particle(p_smoke,projectiles[id].x,projectiles[id].y)
					particlespeed(math.sin(math.rad(angle))*2,-math.cos(math.rad(angle))*2)
					particlecolor(255,255,255)
				end
				-- Free Fire
				fire=objecttable(o_fire)
				for f=1,#fire do
					if math.sqrt((projectiles[id].x-getobjectx(fire[f]))^2+(projectiles[id].y-getobjecty(fire[f]))^2)<=75 then
						removeobject(fire[f])
					end
				end
				-- Free projectile
				freeprojectile(id)
				return 1
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()-30 then
			-- Sound
			playsound(cc.freezegun.sfx_freeze)
			-- Crater
			blue=math.random(200,255)
			if math.random(0,1)==1 then
				terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
			else
				terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,blue,blue,255)
			end
			-- Ice
			terrainimage(cc.freezegun.gfx_ice,projectiles[id].x-15,getwatery()-30)
			terrainimage(cc.freezegun.gfx_ice,projectiles[id].x,getwatery()-30)
			terrainimage(cc.freezegun.gfx_ice,projectiles[id].x+15,getwatery()-30)
			-- Particles
			particle(p_muzzle,projectiles[id].x,projectiles[id].y)
			particlecolor(255,255,255)
			for angle=0,350,10 do
				particle(p_smoke,projectiles[id].x,projectiles[id].y)
				particlespeed(math.sin(math.rad(angle))*2,-math.cos(math.rad(angle))*2)
				particlecolor(255,255,255)
			end
			-- Free Fire
			fire=objecttable(o_fire)
			for f=1,#fire do
				if math.sqrt((projectiles[id].x-getobjectx(fire[f]))^2+(projectiles[id].y-getobjecty(fire[f]))^2)<=75 then
					removeobject(fire[f])
				end
			end
			-- Free projectile
			freeprojectile(id)
			return 1
		end
	end
end